<?php
/**
 * @package     Joomla.Plugin
 * @subpackage  Content.nicepage
 */

defined('_JEXEC') or die;

/**
 * Class PlgContentNicepage
 */
class PlgContentNicepage extends JPlugin
{

    private $_pageView = 'default';
    private $_isNicepageTheme = '0';
    private $_sectionsHtml = '';
    private $_backlink = '';
    private $_styles = '';
    private $_scripts = '';

    /**
     * @param object $context Context
     * @param object $row     Row
     * @param array  $params  Parameters
     * @param int    $page    Page number
     */
    public function onContentPrepare($context, &$row, &$params, $page = 0)
    {
        $app = JFactory::getApplication();

        if ($context !== 'com_content.article' || $app->isAdmin() || !$this->_npInstalled() || (property_exists($row, 'nicepage') && $row->nicepage)) {
            return;
        }

        if (isset($row->id)) {
            $componentPath = dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage';
            JLoader::register('NicepageModelsSections', $componentPath . '/models/sections.php');
            JLoader::register('NicepageHelpersNicepage', $componentPath . '/helpers/nicepage.php');

            $sectionsObj = NicepageModelsSections::getSectionsObject($row->id);
            if ($sectionsObj) {
                $row->nicepage = true;

                $config = NicepageHelpersNicepage::getConfig();

                $props = unserialize(base64_decode($sectionsObj->props));
                $props['pageId'] = $row->id;

                $this->_pageView = isset($props['pageView']) ? $props['pageView'] : '';

                $theme = $app->getTemplate(true);
                $this->_isNicepageTheme = $theme->params->get('nicepagetheme', '0');

                // Change live props to preview props
                $isPreview = JRequest::getBool('isPreview', false);
                $preview = isset($props['preview']) ? $props['preview'] : '';
                if ($isPreview && $preview !== '') {
                    $props = $props['preview'];
                }

                // Backlink processing
                $hideBacklink = isset($config['hideBacklink']) ? (bool)$config['hideBacklink'] : false;
                if ($hideBacklink) {
                    $props['backlink'] = str_replace('u-backlink', 'u-backlink u-hidden', $props['backlink']);
                }

                // Start nicepage content processing
                $currentText = $row->text;
                $currentPostId = $row->id;
                $row->text = $props['publishHtml'];
                $row->id = '-1';
                JPluginHelper::importPlugin('content');
                $dispatcher = JEventDispatcher::getInstance();
                $dispatcher->trigger('onContentPrepare', array($context, &$row, &$params, 0));
                $props['publishHtml'] = $row->text;
                $row->text = $currentText;
                $row->id = $currentPostId;
                // End nicepage content processing

                // Redirect to landing page
                if ($this->_pageView == 'landing') {
                    $app->set('theme', 'landing');
                    $app->set('themes.base', JPATH_ADMINISTRATOR . '/components/com_nicepage/views');
                    $app->set('themeFile', 'landing.php');
                    $app->set('props', $props);
                    $app->set('editLinkHtml', $this->getEditLinkHtml($params, $row->id));
                    return;
                }

                $this->_addPageProperties($props);
                $this->_setScripts($config);
                $this->_setStyles($props, $config);
                $this->_setBacklink($props);
                $this->_setSectionsHtml($props);

                $row->introtext = $row->text = '<!--np_content-->' . $this->_getSectionsHtml() . $this->getEditLinkHtml($params, $row->id) . '<!--/np_content-->';
            }
        }
        return;
    }

    /**
     * Get edit link html
     *
     * @param JRegistry $params
     * @param int       $id
     *
     * @return string
     */
    function getEditLinkHtml($params, $id) {
        $html = '';
        $adminUrl = JURI::root(true) . '/administrator';
        $icon = $adminUrl . '/components/com_nicepage/assets/images/button-icon.png';
        $link = $adminUrl . '/index.php?option=com_nicepage&autostart=1&postid=' . $id;
        if ($params->get('access-edit')) {
            $html= <<<HTML
        <a href="$link" target="_blank" class="edit-nicepage-button">Edit Page</a>
        <style>
            a.edit-nicepage-button {
                position: fixed;
                top: 0;
                right: 0;
                background: url($icon) no-repeat 5px 6px;
                background-size: 16px;
                color: #4184F4;
                font-family: Georgia;
                margin: 10px;
                display: inline-block;
                padding: 5px 5px 5px 25px;
                font-size: 14px;
                line-height: 18px;
                background-color: #fff;
                border-radius: 3px;
                border: 1px solid #eee;
                z-index: 9999;
                text-decoration: none;
            }
            a.edit-nicepage-button:hover {
                color: #BC5A5B;
            }
        </style>
HTML;
        }
        return $html;
    }
    /**
     *  Proccess page content after rendering
     */
    public function onAfterRender()
    {
        $app = JFactory::getApplication();

        if ($app->isAdmin()) {
            return;
        }

        $pageContent = $app->getBody();

        if (preg_match('/<\!--np\_content-->([\s\S]+?)<\!--\/np\_content-->/', $pageContent, $matches)) {
            $sectionsHtml = $matches[1];
            if ($sectionsHtml && $this->_pageView === 'landing_with_header_footer') {
                $bodyStartTag = '<body>';
                $bodyContent = '';
                $bodyEndTag = '</body>';
                if (preg_match('/(<body[^>]+>)([\s\S]*)(<\/body>)/', $pageContent, $matches)) {
                    $bodyStartTag = $matches[1];
                    $bodyContent = trim($matches[2]);
                    $bodyEndTag = $matches[3];
                }

                if ($bodyContent == '') {
                    $newPageContent = $bodyStartTag . $sectionsHtml . $bodyEndTag;
                } else {
                    $newPageContent = $bodyStartTag;
                    if (preg_match('/<header[^>]+>[\s\S]*<\/header>/', $bodyContent, $matches2)) {
                        $newPageContent .= $matches2[0];
                    }
                    $newPageContent .= $sectionsHtml;
                    if (preg_match('/<footer[^>]+>[\s\S]*<\/footer>/', $bodyContent, $matches3)) {
                        $newPageContent .= $matches3[0];
                    }
                    $newPageContent .= $bodyEndTag;
                }
                $pageContent = preg_replace('/(<body[^>]+>)([\s\S]*)(<\/body>)/', '[[body]]', $pageContent);
                $pageContent = str_replace('[[body]]', $newPageContent, $pageContent);
            } else {
                $pageContent = preg_replace('/<!--\/?np\_content-->/', '', $pageContent);
            }
        }
        if ($app->input->get('toEdit', '0') === '1') {
            $pageContent = preg_replace('/class="(item-page|u-page-root)/', ' id="np-test-container" class="$1', $pageContent);
        }
        $pageContent = str_replace('</head>', $this->_getStyles() . '</head>', $pageContent);
        $pageContent = str_replace('</head>', $this->_getScripts() . '</head>', $pageContent);
        $pageContent = str_replace('</body>', $this->_getBacklink() . '</body>', $pageContent);
        if (strpos($pageContent, '<meta name="viewport"') === false) {
            $pageContent = str_replace('<head>', '<head>' . '<meta name="viewport" content="width=device-width, initial-scale=1.0" />', $pageContent);
        }
        $app->setBody($pageContent);
    }

    /**
     * Check on exist nicepage plugin
     *
     * @return bool
     */
    private function _npInstalled()
    {
        if (!file_exists(dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage')) {
            return false;
        }
        if (!JComponentHelper::getComponent('com_nicepage', true)->enabled) {
            return false;
        }
        return true;
    }

    /**
     * Set sections html
     *
     * @param array $props Page properties
     */
    private function _setSectionsHtml($props)
    {
        $assets = JURI::root(true) . '/administrator/components/com_nicepage/assets';
        $responsiveScript = <<<SCRIPT
<script>
    (function ($) {
        var ResponsiveCms = window.ResponsiveCms;
        if (!ResponsiveCms) {
            return;
        }
        ResponsiveCms.contentDom = $('script:last').parent();
    	ResponsiveCms.mediaStyleUrl = "$assets/css/media.css";
        ResponsiveCms.responsiveStyleUrl = "$assets/css/responsive.css";
        
        if (typeof ResponsiveCms.recalcClasses === 'function') {
            ResponsiveCms.recalcClasses();
        }
    })(jQuery);
</script>
SCRIPT;
        $this->_sectionsHtml = $responsiveScript . NicepageModelsSections::processSectionsHtml($props['publishHtml']);

        if ($this->_isNicepageTheme !== '1') {
            $this->_sectionsHtml = '<div class="nicepage-container"><div style="' . $props['bodyStyle'] . '" class="'. $props['bodyClass'] .'">' . $this->_sectionsHtml . '</div></div>';
        } else {
            $bodyScript = <<<SCRIPT
<script>
var body = document.body;
    
    body.className += " {$props['bodyClass']}";
    body.style.cssText += " {$props['bodyStyle']}";
</script>
SCRIPT;
            $this->_sectionsHtml = $bodyScript . $this->_sectionsHtml;
        }
    }

    /**
     * Get sections html
     *
     * @return string
     */
    private function _getSectionsHtml()
    {
        return $this->_sectionsHtml;
    }

    /**
     * Set page backlink
     *
     * @param array $props Page properties
     */
    private function _setBacklink($props)
    {
        $backlink = isset($props['backlink']) ?  $props['backlink'] : '';
        if ($backlink && $this->_pageView == 'default') {
            if ($this->_isNicepageTheme !== '1') {
                $backlink = '<div class="nicepage-container"><div class="'. $props['bodyClass'] .'">' . $backlink . '</div></div>';
            } else {
                $backlink = '';
            }
        }
        $this->_backlink = $backlink;
    }

    /**
     * Get page backlink
     *
     * @return string
     */
    private function _getBacklink()
    {
        return $this->_backlink;
    }

    /**
     * Add custom page properties
     *
     * @param array $props Page properties
     */
    private function _addPageProperties($props)
    {
        $document = JFactory::getDocument();
        if (isset($props['titleInBrowser']) && $props['titleInBrowser'] != '') {
            $document->setTitle($props['titleInBrowser']);
        }
        if (isset($props['description']) && $props['description'] != '') {
            $document->setDescription($props['description']);
        }
        if (isset($props['keywords']) && $props['keywords'] != '') {
            $document->setMetadata('keywords', $props['keywords']);
        }
        if (isset($props['metaTags']) && $props['metaTags'] != '') {
            $document->addCustomTag($props['metaTags']);
        }
        if (isset($props['customHeadHtml']) && $props['customHeadHtml'] != '') {
            $document->addCustomTag($props['customHeadHtml']);
        }
    }

    /**
     * Set plugin styles
     *
     * @param array $props  Page properties
     * @param array $config Plugin parameters
     */
    private function _setStyles($props, $config)
    {
        $pageCssUsedIds = isset($props['pageCssUsedIds']) ? $props['pageCssUsedIds'] : '';
        $siteStyleCss = NicepageHelpersNicepage::buildSiteStyleCss($config, $pageCssUsedIds, $props['publishHtml'], $props['pageId']);

        $sectionsHead = $props['head'];
        $autoResponsive = isset($config['autoResponsive']) ? !!$config['autoResponsive'] : true;
        $assets = JURI::root(true) . '/administrator/components/com_nicepage/assets';
        if ($autoResponsive && $this->_isNicepageTheme == '0') {
            $sectionsHead = preg_replace('#\/\*RESPONSIVE_MEDIA\*\/([\s\S]*?)\/\*\/RESPONSIVE_MEDIA\*\/#', '', $sectionsHead);
            $this->_styles .= '<link href="' . $assets . '/css/responsive.css" rel="stylesheet">';
        } else {
            $sectionsHead = preg_replace('#\/\*RESPONSIVE_CLASS\*\/([\s\S]*?)\/\*\/RESPONSIVE_CLASS\*\/#', '', $sectionsHead);
            if ($this->_isNicepageTheme == '0') {
                $this->_styles .= '<link href="' . $assets . '/css/media.css" rel="stylesheet">';
            }
        }
        $allCss = $siteStyleCss . $sectionsHead;
        if ($this->_isNicepageTheme !== '1') {
            ob_start();
            include_once dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage/assets/css/nicepage.css';
            $allCss = ob_get_clean() . $allCss;
            $allCss = preg_replace_callback(
                '/([^{}]+)\{[^{}]+?\}/',
                function ($match) {
                    $selectors = $match[1];
                    $parts = explode(',', $selectors);
                    $newSelectors = implode(
                        ',',
                        array_map(
                            function ($part) {
                                if (!preg_match('/html|body|sheet/', $part)) {
                                    return ' .nicepage-container ' . $part;
                                } else {
                                    return $part;
                                }
                            },
                            $parts
                        )
                    );
                    return str_replace($selectors, $newSelectors, $match[0]);
                },
                $allCss
            );

        }
        $this->_styles .= isset($props['fonts']) ? $props['fonts'] : '';
        $this->_styles .= '<style id="nicepage-style-css">' . $allCss . '</style>';
    }

    /**
     * Get plugin styles
     *
     * @return string
     */
    private function _getStyles()
    {
        return $this->_styles;
    }

    /**
     * Set plugin scripts
     *
     * @param array $config Plugin parameters
     */
    private function _setScripts($config)
    {
        if ($this->_isNicepageTheme !== '1') {
            $assets = JURI::root(true) . '/administrator/components/com_nicepage/assets';
            if (isset($config['jquery']) && $config['jquery'] == '1') {
                $this->_scripts .= '<script src="' . $assets . '/js/jquery.js"></script>';
            }
            $this->_scripts .= '<script src="' . $assets . '/js/nicepage.js"></script>';
        }
    }

    /**
     * Get plugin scripts
     *
     * @return string
     */
    private function _getScripts()
    {
        return $this->_scripts;
    }
}