<?php
/**
 * @package     Joomla.Plugin
 * @subpackage  Editors-xtd.nicepage
 */

defined('_JEXEC') or die;

class PlgButtonNicepage extends JPlugin
{
    private $_adminUrl = '';
    private $_domain = '';

    private $_article = null;
    private $_sections = null;
    private $_isConvertRequired = null;

    private $_scripts = '';
    /**
     * PlgButtonNicepage constructor.
     *
     * @param object $subject Subject object
     * @param object $config  Config object
     */
    public function __construct(& $subject, $config)
    {
        parent::__construct($subject, $config);
        $this->loadLanguage();

        if ($this->isDisplayed()) {
            $this->updateDuplicatedPageId();
            $this->setAdminUrl();
            $this->setDomain();
            $aid = JFactory::getApplication()->input->get('id', '');
            $this->_sections = NicepageModelsSections::getSectionsObject($aid);
            $this->_article = JTable::getInstance("content");
            $this->_article->load($aid);
            $this->_isConvertRequired = !$this->_sections && ($this->_article->introtext . $this->_article->fulltext);
        }
    }

    /**
     * @param string $name Name
     */
    public function onDisplay($name)
    {
        if (!$this->_article) {
            return;
        }

        $this->addCommonScript();
        $this->addLinkDialogScript();
        $this->addDataBridgeScript();

        $this->addMainScript();

        $this->includeScripts();

        $this->displayEditButton();
        $this->displayScreenshots();
    }

    /**
     * Check editor button displaying
     *
     * @return bool
     */
    public function isDisplayed()
    {
        $app = JFactory::getApplication();
        $input = $app->input;
        $option = $input->get('option');
        $aid = $input->get('id', '');

        if (!in_array($option, array('com_content')) || '' == $aid || !$app->isAdmin()) {
            return false;
        }

        if (!$this->_npInstalled()) {
            return false;
        }

        $componentPath = dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage';
        JLoader::register('NicepageModelsSections', $componentPath . '/models/sections.php');
        JLoader::register('NicepageHelpersNicepage', $componentPath . '/helpers/nicepage.php');
        JLoader::register('NicepageModelsActions', $componentPath . '/models/actions.php');

        return true;
    }

    /**
     * Update nicepage id after duplicating
     */
    public  function updateDuplicatedPageId()
    {
        $db = JFactory::getDBO();
        $query = $db->getQuery(true);
        $query->update('#__nicepage_sections');
        $query->set($db->quoteName('page_id') . '=' . JFactory::getApplication()->input->get('id', ''));
        $query->where('page_id= 1000000');
        $db->setQuery($query);
        $db->query();
    }

    /**
     * Add main script
     */
    public function addMainScript()
    {
        $input = JFactory::getApplication()->input;
        $aid = $input->get('id', '');
        $start = $input->get('start', '0');
        $autostart = $input->get('autostart', '0');
        $savePageTypeUrl = $this->getAdminUrl() . '/index.php?option=com_nicepage&controller=actions&action=savePageType';
        $duplicatePageUrl = $this->getAdminUrl() . '/index.php?option=com_nicepage&controller=actions&action=duplicatePage';
        $previewPageUrl = dirname($this->getAdminUrl()) . '/index.php?option=com_content&view=article&id=' . $aid;
        $userId = JFactory::getUser()->id;
        $frontUrl = dirname($this->getAdminUrl()) . '/index.php?option=com_nicepage';

        $themeTypographyCacheForceRefresh = '0';
        $cachedDefaultTheme = isset($_COOKIE['DEFAULT_THEME']) ? $_COOKIE['DEFAULT_THEME'] : '';
        if (!$cachedDefaultTheme || $cachedDefaultTheme !== $this->getDefaultTemplate()) {
            setcookie('DEFAULT_THEME', $this->getDefaultTemplate(), time() + 31536000); // will expire after year
            $themeTypographyCacheForceRefresh = '1';
        }

        $pageView = 'landing';
        if ($this->_sections) {
            $props = unserialize(base64_decode($this->_sections->props));
            $pageView = isset($props['pageView']) ? $props['pageView'] : $pageView;
        }

        switch($pageView) {
        case 'landing':
            $templateOptions = JText::sprintf('PLG_EDITORS-XTD_TEMPLATE_OPTIONS', '', 'selected', '');
            break;
        case 'landing_with_header_footer':
            $templateOptions = JText::sprintf('PLG_EDITORS-XTD_TEMPLATE_OPTIONS', '', '', 'selected');
            break;
        default:
            $templateOptions = JText::sprintf('PLG_EDITORS-XTD_TEMPLATE_OPTIONS', 'selected', '', '');
        }
        $buttonText = $this->_isConvertRequired ? JText::_('PLG_EDITORS-XTD_TURN_TO_NICEPAGE_BUTTON_TEXT') : JText::_('PLG_EDITORS-XTD_EDIT_WITH_NICEPAGE_BUTTON_TEXT');
        $buttonAreaClass = $this->_isConvertRequired ? '' : 'nicepage-select-template-area';

        $this->_scripts .= <<<EOF

jQuery(function($) {
    
    var dataInfo = dataBridge.getInfo();
    if (!dataInfo['themeColorScheme']) {
        var loadCallback;
        var forceRefresh = '$themeTypographyCacheForceRefresh';
        var needResetCache = !localStorage.np_theme_typography_cache ||
            forceRefresh === '1';
    
        if (needResetCache) {
            delete localStorage.np_theme_typography_cache;
        }
        
        window.loadAppHook = function (load) {
            if (localStorage.np_theme_typography_cache) {
                jQuery.extend(dataBridgeData.info, JSON.parse(localStorage.np_theme_typography_cache));
                console.log('Regular load app.js');
                load();
                return;
            }
            loadCallback = load;
        };
        var iframe = $('<iframe>', { id: 'np-loader', 'style': 'position:absolute, visibility: hidden, width: 1800px; height:0; border:0;' }),
            nav = $('nav');
        
        if (!nav.length) {
            nav = $('div[id="nav"]');
        }
        
        nav.before(iframe);
        
        var loaderIframe = document.getElementById('np-loader');
        loaderIframe.addEventListener("load", function() {
            localStorage.np_theme_typography_cache = JSON.stringify(NpTypographyParser.parse(loaderIframe));
            $(loaderIframe).remove();
            console.log('Typography cache updated');
            $.extend(dataBridgeData.info, JSON.parse(localStorage.np_theme_typography_cache));
            if (loadCallback) {
                console.log('Deferred load app.js');
                loadCallback();
            }
        });
        
        if (location.protocol === "https:" && dataBridgeData.info.typographyPageHtmlUrl.indexOf('http://') !== -1) {
            console.log('Regular load app.js due to CORS');
            delete window.loadAppHook;
        } else {
            loaderIframe.src = dataBridgeData.info.typographyPageHtmlUrl;
        }
    }
            
    // autostart nicepage from cms admin main menu
    if ('$start' == '1' || '$autostart' == '1') {
        runNicepage();
    }
    var nicepageButton = $('<a href="#" class="btn nicepage-button">$buttonText</a>'),
        nicepageArea = $('<div class="$buttonAreaClass"></div>');
    
    nicepageArea.append(nicepageButton);
    
    if ($('form').length) {
        $('form').before(nicepageArea);
    } else {
        $('section[id="content"]').prepend(nicepageArea);
    }
    
    nicepageButton.click(function(e) {
        e.preventDefault();
        runNicepage();
    });
    if ('$buttonAreaClass' !== '') {
        Joomla.originalsubmitbutton = Joomla.submitbutton; 
        Joomla.submitbutton = function npsubmitbutton(action) {
            if (action && action == 'article.save2copy') {
                $.post('$duplicatePageUrl', { postId: '$aid' }).done(function(data) {
                    if (data && data.indexOf('ok') !== -1) {
                        Joomla.originalsubmitbutton(action);
                    }
                }, 'json');
            } else {
                Joomla.originalsubmitbutton(action);
            }
            
        }
       
        var selectObj = $('$templateOptions');
        nicepageButton.after(selectObj);
        $('#toolbar-apply button, #toolbar-save button').click(function() {
            var pageType = $('.nicepage-select-template').val();
            sendRequest({
                url: '$savePageTypeUrl',
                method: 'POST',
                data: {
                    pageType: pageType,
                    pageId: $aid
                }
            }, function (error, response) {
                if (error) {
                    console.error(e);
                    alert('Save page type error.');
                }
            });
        });
    }
    
    $.post('$frontUrl', { uid: '$userId' }).done(function( data ) {
        if (data && data.indexOf('ok') !== -1) {
            var nicepagePreviewButton = $('<a href="$previewPageUrl" target="_blank" class="btn nicepage-preview-button">Preview page</a>');
            nicepageButton.after(nicepagePreviewButton);
        }
    }, 'json');
});
EOF;
    }

    /**
     * Add common scripts
     */
    public function addCommonScript()
    {
        $domain = $this->getDomain();
        $aid = JFactory::getApplication()->input->get('id', '');
        $currentUrl = $this->getAdminUrl() . '/index.php?option=com_nicepage&postid=' . $aid . ($domain ? '&domain=' . $domain : '');

        JHtml::_('behavior.modal'); // for SqueezeBox

        $this->_scripts .= <<<EOF
function runNicepage()
{
    (function($){
        var iframe = $('<iframe>', {
                src: '$currentUrl',
                id: 'editor-frame'
            }),
            nav = $('nav');
        
        if (!nav.length) {
            nav = $('div[id="nav"]');
        }
        
        nav.after(iframe);
        iframe.css('height', 'calc(100vh - ' + nav.height() + 'px)');
        iframe.css('width', '100%');

        $(document).scroll(function() {
            $(this).scrollTop(0);
        });

        $('body').addClass('editor');
    })(jQuery);
}

SqueezeBox.extend({
    applyContent: function(content, size) {
        if (!this.isOpen && !this.applyTimer) return;
        this.applyTimer = clearTimeout(this.applyTimer);
        this.hideContent();
        if (!content) {
            this.toggleLoading(true);
        } else {
            if (this.isLoading) this.toggleLoading(false);
            this.fireEvent('onUpdate', [this.content], 20);
        }
        if (content) {
            if (['string', 'array'].contains(typeOf(content))) {
                this.content.set('html', content);
            } else if (!(content !== this.content && this.content.contains(content))) {
                this.content.adopt(content);
            }
        }
        this.callChain();
        if (!this.isOpen) {
            this.toggleListeners(true);
            this.resize(size, true);
            this.isOpen = true;
            this.win.setProperty('aria-hidden', 'false');
            this.fireEvent('onOpen', [this.content]);
        } else {
            this.resize(size);
        }
    }
});

function sendRequest(data, callback) {
    var xhr = new XMLHttpRequest();

    function onError() {
        callback(new Error('Failed to send a request to ' + data.url + ' ' + JSON.stringify({
            responseText: xhr.responseText,
            readyState: xhr.readyState,
            status: xhr.status
        }, null, 4)));
    }

    xhr.onerror = onError;
    xhr.onload = function () {
        if (this.readyState === 4 && this.status === 200) {
            callback(null, this.response);
        } else {
            onError();
        }
    };
    xhr.open(data.method || 'GET', data.url);

    if (data.data) {
        var formData = new FormData();
        formData.append("pageType", data.data.pageType);
        formData.append("pageId", data.data.pageId);
        xhr.send(formData);
    } else {
        xhr.send();
    }
}

function postMessageListener(event) {
    if (event.origin !== location.origin) {
        return;
    }
    var data = JSON.parse(event.data);
    if (!data) {
        return;
    }
    if (data.action === 'close') {
        window.location.href = data.closeUrl;
    } else if (data.action === 'editLinkDialogOpen') {
    	window.dataForDialog = data.data;
        openEditLinkDialog(data.data);
    }
}

if (window.addEventListener) {
    window.addEventListener("message", postMessageListener);
} else {
    window.attachEvent("onmessage", postMessageListener); // IE8
}
EOF;
    }

    /**
     * Add script for making data for editor
     */
    public function addDataBridgeScript()
    {
        $app = JFactory::getApplication();
        $input = $app->input;
        $aid = $this->_article->id;
        $start = $input->get('start', '0');
        $autostart = $input->get('autostart', '0');
        $domain = $this->getDomain();

        $editorSettings = NicepageHelpersNicepage::getEditorSettings();
        $editorSettings['pageId'] = $this->_isConvertRequired ? '' : $aid;
        $editorSettings['startPageId'] = $aid;

        $cmsSettings = NicepageHelpersNicepage::getCmsSettings();
        $cmsSettings['isFirstStart'] = $start == '1' ? true : false;

        $editorSettingsJson = json_encode($editorSettings, JSON_PRETTY_PRINT);
        $cmsSettingsJson = json_encode($cmsSettings, JSON_PRETTY_PRINT);

        $modelActions = new NicepageModelsActions();
        $site = $modelActions->getSite();
        if ($this->_article->state == '2' && ($start == '1' || $autostart == '1')) {
            $site['items'][] = array(
                'siteId' => '1',
                'title' => '',
                'id' => (string) $aid,
                'order' => 0,
                'status' => 2,
                'editorUrl' => $this->getAdminUrl() . '/index.php?option=com_nicepage&autostart=1&postid=' . $aid . ($domain ? '&domain=' . $domain : ''),
                'htmlUrl' => $this->getAdminUrl() . '/index.php?option=com_nicepage&controller=actions&action=getPageHtml&pageId=' . $aid
            );
        }
        $info = array(
            'productsExists' => $this->vmEnabled(),
            'newPageUrl' => $this->getAdminUrl() . '/index.php?option=com_nicepage&dashboard=1' . ($domain ? '&domain=' . $domain : ''),
            'forceModified' => $this->forceModified(),
            'generalSettingsUrl' => $this->getAdminUrl() . '/index.php?option=com_config#page-server',
            'typographyPageHtmlUrl' => $this->getFrontendUrl()
        );
        $themeEditorSettings = $this->getEditorSettingsFromDefaultTheme();
        if ($themeEditorSettings) {
            $info['themeTypography'] = $themeEditorSettings['typography'];
            $info['themeFontScheme'] = $themeEditorSettings['fontScheme'];
            $info['themeColorScheme'] = $themeEditorSettings['colorScheme'];
        }

        $data = json_encode(
            array (
                'site' => $site,
                'pageHtml' => $modelActions->getPageHtml($aid),
                'startTerm' => $this->_isConvertRequired ? 'site:joomla:' . $aid : '',
                'info' => $info,
                'nicePageCss' => $this->getDynamicNicepageCss()
            ),
            JSON_PRETTY_PRINT
        );

        $this->_scripts .= <<<EOF
var dataBridgeData = $data;
window.dataBridge = {
    getSite: function () {
        return dataBridgeData.site;
    },
    getPageHtml: function () {
        return dataBridgeData.pageHtml;
    },
    getStartTerm: function () {
        return dataBridgeData.startTerm;
    },
    getInfo: function getInfo() {
        return dataBridgeData.info;
    },
    getNicePageCss: function getNicePageCss() {
        return dataBridgeData.nicePageCss;
    },
    settings: $editorSettingsJson,
    cmsSettings: $cmsSettingsJson
};
EOF;
    }

    /**
     * Check force saving or not
     */
    public function forceModified()
    {
        if ($this->_sections) {
            $props = unserialize(base64_decode($this->_sections->props));
            return isset($props['pageCssUsedIds']) ? false : true;
        }
        return true;
    }

    /**
     * Add joomla link dialog script
     */
    public function addLinkDialogScript()
    {
        $editLinkUrl = $this->getAdminUrl() . '/index.php?option=com_content&view=articles&layout=modal&tmpl=component';
        $customUrlOptions = <<<HTML
<style>
.custom-url-options {
    width:100%;
}
.custom-url-options label{
    width: 60px;
    display: inline-block;
}
.custom-url-options input[type=text]{
    width: 350px;
}
.custom-url-options:after {
    content: "";
    clear: both;
    display: table;
}
.link-destination,
.target-option {
    margin-left: 70px;
}
.link-destination {
    margin-top: 4px;
}
.link-destination input,
.target-option input {
    margin-right: 10px;
    margin-top: 0px;
}
.link-destination .link-destination-label {
    width: auto;
    display: inline-block;
    vertical-align: top;
    margin-left: -80px;
    margin-top: 4px;
    width: 76px;
}
.link-destination ul {
    list-style-type: none;
    margin-left: 0px;
    margin-top: 4px;
    display: inline-block;
}
.anchors-list {
    background-color: #F5F5F5;
    border: 1px solid #BFBFBF;
    padding: 4px 6px 4px 10px;
    margin: 10px auto auto 70px;
    list-style-type: none;
}
.anchors-list li {
    cursor: pointer;
}
.anchors-list li:hover,
.anchors-list li.selected {
    background-color: #e5f2ff;
}
.anchors-list li a {
    color: #666;
}
</style>
<div class="custom-url-options">
    <div style="float:left">
        <div class="caption-option"><label for="caption">{{caption}}</label><input type="text" name="caption" value="" /></div>
        <div class="url-option"><label for="url">{{url}}</label><input type="text" name="url" value="" /></div>
        <div class="target-option"><input type="checkbox" name="target" />{{target}}</div>

        <div class="link-destination hidden">
        <div class="link-destination-label">{{Destination}}</div>
            <ul>
                <li><input type="radio" name="link-destination" id="page-link" value="page"/><label for='page-link'>{{pageLink}}</lable></li>
                <li><input type="radio" name="link-destination" id="anchor-link" value="section"/><label for='anchor-link'>{{anchorLink}}</lable></li>
            </ul>
        </div>      

        <ul class="anchors-list hidden" id="anchors-list"></ul>
    </div>
    <div style="float:right">
        <button type="button" class="btn btn-success" id="save-options">Save</button>
    </div>
</div>
HTML;
        $customUrlOptions = base64_encode($customUrlOptions);

        $this->_scripts .= <<<EOF
function openEditLinkDialog(data) {
    //override joomla action for modal close
    window.jModalClose = function () {};
    window.jSelectArticle = function () {};
    (function($) {
        var editorFrame = $('#editor-frame')[0].contentWindow;
        var editLinkUrl = '$editLinkUrl' + '&u=' + Date.now();

        function isUrlToAnchor(url) {
            return url && url.indexOf('#') === 0 && url !== '#';
        }

        function selectItemByUrl(list, url) {
            var matchUrl = function (index, element) {
                return $(element).children('a').attr('href') === url;
            }
            var listItems = list.find('li'),
                matchItem = listItems.filter(matchUrl);
            if (matchItem.hasClass('selected')) {
                return;
            }
            listItems.removeClass('selected');
            matchItem.addClass('selected');
        }

        function showAnchorsList(dialog) {
            var urlField = dialog.find('.url-option input'),
                url = urlField.val(),
                anchorsList = dialog.find('#anchors-list');
            anchorsList.removeClass('hidden');
            selectItemByUrl(anchorsList, url);
        }

        function hideAnchorsList(dialog) {
            var anchorsList = dialog.find('#anchors-list');
            anchorsList.addClass('hidden');
        }
		
        function showPagesList(dialog) {
            var pagesList = dialog.find('#adminForm');
            pagesList.removeClass('hidden');
        }

        function hidePagesList(dialog) {
            var pagesList = dialog.find('#adminForm');
            pagesList.addClass('hidden');
        }        

        function handleAnchorListItemClick(event) {
            event.preventDefault();
            var listItem = $(event.target),
                list;
            if (listItem.is('li')) {
                listItem = listItem.children();
            }
            if (!listItem.is('.anchor-link') || listItem.length !== 1) {
                return;
            }
            list = listItem.closest('ul');
            var value = listItem.attr('href'),
                customUrlOptions = listItem.closest('.custom-url-options'),
                urlField = customUrlOptions.find('.url-option input');
            urlField.val(value ? value : '');
            selectItemByUrl(list, value);
        }

        function handleLinkDestinationChange(event) {
            var target = $(event.target),
                customUrlOptions = target.closest('.custom-url-options'),
                dialog = customUrlOptions.parent(),
                value = target.attr('value');
            if (value === 'section') {
                showAnchorsList(dialog);
                hidePagesList(dialog)
            } else if (value === 'page') {
                showPagesList(dialog);
                hideAnchorsList(dialog);
            }
            window.dataForDialog.customizeLinkType = value; 
        }

        function createAnchorListItem(anchor) {
            if (!anchor) {
                return anchor;
            }
            var result = $('<li><a class="anchor-link" href=""></a></li>');
            result.children('.anchor-link').attr('href', anchor.url);
            result.children('.anchor-link').text(anchor.title);
            return result;
        }

        function appendAnchorLinks(list, anchors) {
            if (!anchors || !anchors.length) {
                return;
            }
            var listItems = $.map(anchors, createAnchorListItem);
            $.each(listItems, function (i, item) {
                list.append(item);
            });
        }
		
        SqueezeBox.fromElement(editLinkUrl, {
            size : {x : 800, y : 500},
            iframePreload: true,
            handler : 'iframe',
            onUpdate : function (container) {
                if (container.firstChild) {
                    $('.sbox-content-iframe').css('display', '');
                    var ifrDoc = container.firstChild.contentDocument;
                    var data = window.dataForDialog;
                    if (!$('.custom-url-options', ifrDoc).length && data) {
                        var customHtml = atob('$customUrlOptions').replace(/\{\{(\w+?)\}\}/g, function(str, p1) {
                            return data.l[p1];
                        });
                        $('body', ifrDoc).prepend(customHtml);
                        $('.caption-option input', ifrDoc).val(data.value ? data.value : 'Text');
                        $('.url-option input', ifrDoc).val(data.url ? data.url : '#');
                        $('.target-option input', ifrDoc).prop('checked', data.blank);

                        $('#anchors-list', ifrDoc).empty();
                        if (data.anchorsList && data.anchorsList.length) {
                            appendAnchorLinks($('#anchors-list', ifrDoc), data.anchorsList);
                            $('.link-destination', ifrDoc).removeClass('hidden');
                            var linkType = data.customizeLinkType ? data.customizeLinkType : (isUrlToAnchor(data.url) ? 'section' : 'page');   
                            if (linkType === 'section') {
                                $('#anchor-link', ifrDoc).prop('checked', true);
                                showAnchorsList($(ifrDoc));
                                hidePagesList($(ifrDoc));
                            } else {
                                $('#page-link', ifrDoc).prop('checked', true);
                                showPagesList($(ifrDoc));
                                hideAnchorsList($(ifrDoc));
                            }
                        }
                    }
                    $('#adminForm table tbody a', ifrDoc).on('click', function() {
                        var uriAttr = $(this).attr('data-uri') || $(this).attr('onclick'),
                            url = uriAttr.match(/index.php[^,'"]+/),
                            text = $(this).text().trim();
                        $('.caption-option input', ifrDoc).val(text);
                        $('.url-option input', ifrDoc).val(url ? url[0] : '');
                    });
                    
                    $('#save-options', ifrDoc).on('click', function() {
                        $(ifrDoc).data('close-after-save', true);
                        editorFrame.postMessage(JSON.stringify({
                            action: 'editLinkDialogClose',
                            data: {
                                value: $('.caption-option input', ifrDoc).val(),
                                url: $('.url-option input', ifrDoc).val(),
                                blank: $('.target-option input', ifrDoc).is(":checked") ? true : false
                            }
                        }), window.location.origin);
                        SqueezeBox.close();
                    });

                    $('.anchors-list', ifrDoc).on('click', handleAnchorListItemClick);
                    $('.link-destination input', ifrDoc).on('change', handleLinkDestinationChange);
                }
            },
            onClose : function (container) {
                if (!$(container.firstChild.contentDocument).data('close-after-save'))
                    editorFrame.postMessage(JSON.stringify({action: 'editLinkDialogClose'}), window.location.origin);
            }
        });
    })(jQuery);
}            
EOF;

    }

    /**
     * Include all scripts to page document
     */
    public function includeScripts()
    {
        $doc = JFactory::getDocument();

        $parserUrl = $this->getAdminUrl() . '/components/com_nicepage/assets/js/typography-parser.js';
        $doc->addScript($parserUrl);

        $doc->addScriptDeclaration($this->_scripts);
    }

    /**
     * Display nicepage editor button
     */
    public function displayEditButton()
    {
        $input = JFactory::getApplication()->input;
        $template = JFactory::getApplication()->getTemplate();
        $start = $input->get('start', '0');
        $autostart = $input->get('autostart', '0');
        $favicon = $this->getAdminUrl() . '/components/com_nicepage/assets/images/button-icon.png';
        $cssDisplay = ($start == '1' || $autostart == '1') ? 'none' : 'block';
        $css = <<<EOF
body.editor>*:not(nav) {
    display:none;
}

body>*:not(nav) {
    display:$cssDisplay;
}
EOF;
        if ($template == 'hathor') {
            $css = <<<EOF
body.editor>#containerwrap>*:not([id="nav"]) {
    display:none;
}

body>#containerwrap>*:not([id="nav"]) {
    display:$cssDisplay;
}
EOF;
        }

        $css .= <<<EOF
#np-loader {
	display: block !important;
}

#editor-frame{
    display: block !important;
}
#sbox-overlay, #sbox-window  {
    display: block;
}
.btn.nicepage-button, .btn.nicepage-preview-button{
    color: #4184F4;
    font-weight: bold;
    font-family: Arial;
    margin: 10px 10px 20px 0px;
}

.btn.nicepage-button {
    padding-right: 5px;
    padding-left: 25px;
    background: url('$favicon') no-repeat 4px 4px;
    background-size: 16px;
}

.btn.nicepage-preview-button {
    background: url('$favicon') no-repeat -9999px;
}

.nicepage-select-template {
    margin-left: 10px
}
.nicepage-select-template-area {
    border: 1px solid #ddd;
    margin: 10px 0 20px 0;
    padding: 15px;
}
EOF;
        JFactory::getDocument()->addStyleDeclaration($css);
    }

    /**
     * Display screenshot of article, created from sections
     */
    public function displayScreenshots()
    {
        if (!$this->_sections) {
            return;
        }

        $content = NicepageModelsSections::getSectionsScreenshots($this->_sections);
        $css = <<<EOF
fieldset.adminform {
    display:none;
}
.nicepage-button {
    margin: 20px;
}
#preview-container {
    overflow: hidden;
}
#preview-frame {
    transform: scale(0.3);
    transform-origin: 0 0;
    height: 333.333%;
}
EOF;
        $js = <<<EOF
function renderScreenshots()
{
    (function($){
        var previewContainer = $('<div>', {
            id: 'preview-container'
        }),
        previewFrame = $('<iframe>', {
            id: 'preview-frame',
            frameborder: 0,
            scrolling: 'no',
            width: '9999px'
        });

        $('.adminform').before(previewContainer);
        previewContainer.append(previewFrame);
       
        var doc = previewFrame[0].contentDocument;
        doc.open();
        doc.write($content);
        doc.close();
        
        previewFrame.load(function() {
            var containerHeight = 0;
            $('section', previewFrame.contents()).each(function(i, el) {
                containerHeight += $(el).height();
            });
            previewContainer.height(containerHeight * 0.3);
        });

        $('body', previewFrame.contents()).click(function(e) {
            runNicepage();
        });
    })(jQuery);
}

jQuery(function($) {
    renderScreenshots();
});
EOF;
        $doc = JFactory::getDocument();
        $doc->addScriptDeclaration($js);
        $doc->addStyleDeclaration($css);
    }

    /**
     * Check the existence of Virtuemart
     *
     * @return bool
     */
    public function vmEnabled()
    {
        if (!file_exists(dirname(JPATH_ADMINISTRATOR) . '/components/com_virtuemart/')) {
            return false;
        }

        if (!JComponentHelper::getComponent('com_virtuemart', true)->enabled) {
            return false;
        }
        return true;
    }

    /**
     * Set domain property
     */
    public function setDomain()
    {
        $this->_domain = JRequest::getVar('domain', (defined('NICEPAGE_DOMAIN') ? NICEPAGE_DOMAIN : ''));
    }

    /**
     * Get domain property
     *
     * @return string
     */
    public function getDomain()
    {
        return $this->_domain;
    }

    /**
     * Set admin url
     */
    public function setAdminUrl()
    {
        $current = dirname(dirname((JURI::current())));
        $this->_adminUrl = $current . '/administrator';
    }

    /**
     * Get admin url
     *
     * @return string
     */
    public function getAdminUrl()
    {
        return $this->_adminUrl;
    }

    /**
     * Get name of default template style
     *
     * @return string
     */
    public function getDefaultTemplate()
    {
        $db = JFactory::getDBO();
        $query = $db->getQuery(true);
        $query->select('*');
        $query->from('#__template_styles');
        $query->where('client_id = \'0\'');
        $query->where('home=1');
        $db->setQuery($query);
        $ret = $db->loadObject();
        return $ret ? $ret->template : '';
    }

    /**
     * Get editor settings from default theme
     *
     * @return mixed|null
     */
    public function getEditorSettingsFromDefaultTheme()
    {
        $template = $this->getDefaultTemplate();
        if ($template) {
            $funcsFilePath = dirname(dirname(JPATH_THEMES)) . '/templates/' . $template . '/template.json';
            if (file_exists($funcsFilePath)) {
                ob_start();
                include_once $funcsFilePath;
                return json_decode(ob_get_clean(), true);
            }
        }
        return null;
    }

    /**
     * Get frontend site url
     *
     * @return string
     */
    public function getFrontendUrl()
    {
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);
        $query->select('*')
            ->from($db->quoteName('#__content'))
            ->where($db->quoteName('state') . ' = 1');
        $db->setQuery($query);
        $ret = $db->loadObject();

        if ($ret !== null) {
            return dirname($this->getAdminUrl()) . '/' . 'index.php?option=com_content&view=article&id=' . $ret->id . '&toEdit=1';
        } else {
            $frontEndUri = new JUri(dirname(dirname((JURI::current()))) . '/');
            $frontEndUri->setVar('toEdit', '1');
            return $frontEndUri->toString();
        }
    }

    /**
     * Get content from nicepage-dynamic.css
     *
     * @return string
     */
    public function getDynamicNicepageCss()
    {
        $assets = dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage/assets';
        ob_start();
        include $assets . '/css/nicepage-dynamic.css';
        return ob_get_clean();
    }

    /**
     * Check on exist nicepage plugin
     *
     * @return bool
     */
    public function _npInstalled()
    {
        if (!file_exists(dirname(JPATH_ADMINISTRATOR) . '/components/com_nicepage')) {
            return false;
        }

        if (!JComponentHelper::getComponent('com_nicepage', true)->enabled) {
            return false;
        }
        return true;
    }
}